@@ -14,7 +14,6 @@ from safe.common.geodesy import Point
 from safe.common.exceptions import InaSAFEError, BoundsError
 from safe.common.polygon import (inside_polygon,
                                  clip_lines_by_polygons, clip_grid_by_polygons)
-#from safe.common.polygon import line_dictionary_to_geometry
 
 from safe.storage.vector import Vector, convert_polygons_to_centroids
 from safe.storage.utilities import geometrytype2string
@@ -38,12 +37,19 @@ def assign_hazard_values_to_exposure_data(hazard, exposure,
              If None (default) the name of the exposure layer is used for
              the returned layer.
        * attribute_name:
-             If hazard layer is of type raster, this is the name for new
+             If hazard layer is of type raster, this will be the name for new
              attribute in the result containing the hazard level.
-             If None (default) the name of hazard is used.
-             If hazard layer is of type vector, it is the name of the
-             attribute to transfer from the hazard layer into the result.
-             If None (default) all attributes are transferred.
+             If None (default) the name of hazard layer is used.
+
+             If hazard layer is polygon and exposure layer raster, this will be
+             the name of the new attribute containing the raster value at each
+             point.
+
+             If hazard and exposure layers are both of type vector,
+             this attribute is ignored.
+
+             If hazard and exposure layers are both of type raster,
+             this attribute is ignored.
         * mode:
              Interpolation mode for raster to point interpolation only
 
@@ -115,8 +121,7 @@ def assign_hazard_values_to_exposure_data(hazard, exposure,
     # Vector-Vector
     elif hazard.is_vector and exposure.is_vector:
         return interpolate_polygon_vector(hazard, exposure,
-                                          layer_name=layer_name,
-                                          attribute_name=attribute_name)
+                                          layer_name=layer_name)
     # Vector-Raster
     elif hazard.is_vector and exposure.is_raster:
         return interpolate_polygon_raster(hazard, exposure,
@@ -167,6 +172,19 @@ def check_inputs(hazard, exposure, layer_name, attribute_name):
     if hazard.is_raster and attribute_name is None:
         layer_name = hazard.get_name()
 
+    if (exposure.is_raster and hazard.is_vector and hazard.is_polygon_data
+        and attribute_name is None):
+        attribute_name = exposure.get_name()
+
+    if (hazard.is_raster and exposure.is_vector and exposure.is_point_data
+        and attribute_name is None):
+        attribute_name = hazard.get_name()
+
+    # Launder for shape files
+    # FIXME (Ole): Remove when (if) we get rid of the shp format
+    if attribute_name is not None:
+        attribute_name = str(attribute_name[:10])
+
     return layer_name, attribute_name
 
 
@@ -230,7 +248,7 @@ def interpolate_raster_vector(source, target,
 
 
 def interpolate_polygon_vector(source, target,
-                               layer_name=None, attribute_name=None):
+                               layer_name=None):
     """Interpolate from polygon vector layer to vector data
 
     Args:
@@ -238,8 +256,6 @@ def interpolate_polygon_vector(source, target,
         * target: Vector data set (points or polygons)  - TBA also lines
         * layer_name: Optional name of returned interpolated layer.
               If None the name of target is used for the returned layer.
-        * attribute_name: Name for new attribute.
-              If None (default) the name of source is used
 
     Output
         I: Vector data set; points located as target with values interpolated
@@ -257,18 +273,15 @@ def interpolate_polygon_vector(source, target,
 
     if target.is_point_data:
         R = interpolate_polygon_points(source, target,
-                                       layer_name=layer_name,
-                                       attribute_name=attribute_name)
+                                       layer_name=layer_name)
     elif target.is_line_data:
         R = interpolate_polygon_lines(source, target,
-                                      layer_name=layer_name,
-                                      attribute_name=attribute_name)
+                                      layer_name=layer_name)
     elif target.is_polygon_data:
         # Use polygon centroids
         X = convert_polygons_to_centroids(target)
         P = interpolate_polygon_points(source, X,
-                                       layer_name=layer_name,
-                                       attribute_name=attribute_name)
+                                       layer_name=layer_name)
 
         # In case of polygon data, restore the polygon geometry
         # Do this setting the geometry of the returned set to
@@ -315,13 +328,8 @@ def interpolate_polygon_raster(source, target,
     # Run underlying clipping algorithm
     polygon_geometry = source.get_geometry(as_geometry_objects=True)
 
-    # FIXME (Ole): Perhaps refactor so that polygon_geometry can
-    # be passed in directly. See issue #324, comment
-    #https://github.com/AIFDR/inasafe/issues/324#issuecomment-9440584
-    # FIXME (Ole): Not sure if needed, but explicitly make sure
-    # exposure raster is *never* scaled here
     polygon_attributes = source.get_data()
-    res = clip_grid_by_polygons(target.get_data(),
+    res = clip_grid_by_polygons(target.get_data(scaling=False),
                                 target.get_geotransform(),
                                 polygon_geometry)
 
@@ -377,12 +385,6 @@ def interpolate_raster_vector_points(source, target,
     verify(target.is_vector)
     verify(target.is_point_data)
 
-    # FIXME (Ole): Why can we not remove this ???
-    # It should now be taken care of in the general input_check above
-    # OK - remove when we leave using the form H.interpolate in impact funcs
-    if layer_name is None:
-        layer_name = target.get_name()
-
     # Get raster data and corresponding x and y axes
     A = source.get_data(nan=True)
     longitudes, latitudes = source.get_geometry()
@@ -397,13 +399,6 @@ def interpolate_raster_vector_points(source, target,
     attributes = target.get_data()
 
     # Create new attribute and interpolate
-    # Remove?
-    N = len(target)
-    if attribute_name is None:
-        attribute_name = source.get_name()
-        # FIXME (Ole): Launder for shape files
-        attribute_name = str(attribute_name[:10])
-
     try:
         values = interpolate_raster(longitudes, latitudes, A,
                                     coordinates, mode=mode)
@@ -416,6 +411,7 @@ def interpolate_raster_vector_points(source, target,
         raise InaSAFEError(msg)
 
     # Add interpolated attribute to existing attributes and return
+    N = len(target)
     for i in range(N):
         attributes[i][attribute_name] = values[i]
 
@@ -426,8 +422,7 @@ def interpolate_raster_vector_points(source, target,
 
 
 def interpolate_polygon_points(source, target,
-                               layer_name=None,
-                               attribute_name=None):
+                               layer_name=None):
     """Interpolate from polygon vector layer to point vector data
 
     Args:
@@ -435,12 +430,14 @@ def interpolate_polygon_points(source, target,
         * target: Vector data set (points)
         * layer_name: Optional name of returned interpolated layer.
               If None the name of target is used for the returned layer.
-        * attribute_name: Name for new attribute.
-              If None (default) the name of source is used
 
     Output
         I: Vector data set; points located as target with values interpolated
         from source
+
+    Note
+        All attribute names from polygons are transferred to the points
+        that are inside them.
     """
 
     msg = ('Vector layer to interpolate to must be point geometry. '
@@ -453,16 +450,7 @@ def interpolate_polygon_points(source, target,
     verify(layer_name is None or
            isinstance(layer_name, basestring), msg)
 
-    msg = ('Attribute must be either a string or None. I got %s'
-           % (str(type(target)))[1:-1])
-    verify(attribute_name is None or
-           isinstance(attribute_name, basestring), msg)
-
     attribute_names = source.get_attribute_names()
-    if attribute_name is not None:
-        msg = ('Requested attribute "%s" did not exist in %s'
-               % (attribute_name, attribute_names))
-        verify(attribute_name in attribute_names, msg)
 
     #----------------
     # Start algorithm
@@ -484,29 +472,14 @@ def interpolate_polygon_points(source, target,
 
     # Augment point features with empty attributes from polygon
     for a in attributes:
-        if attribute_name is None:
-            # Use all attributes
-            for key in attribute_names:
-                a[key] = None
-        else:
-            # Use only requested attribute
-            # FIXME (Ole): Test for this is not finished
-            # BUT don't do it....Issue #251
-            a[attribute_name] = None
-
-        # Always create default attribute flagging if point was
-        # inside any of the polygons
-        # FIXME (Ole): Remove as already appended.....
-        a[DEFAULT_ATTRIBUTE] = None
+        # Create all attributes that exist in source
+        for key in attribute_names:
+            a[key] = None
 
     # Traverse polygons and assign attributes to points that fall inside
     for i, polygon in enumerate(geom):
-        if attribute_name is None:
-            # Use all attributes
-            poly_attr = data[i]
-        else:
-            # Use only requested attribute
-            poly_attr = {attribute_name: data[i][attribute_name]}
+        # Carry all attributes across from source
+        poly_attr = data[i]
 
         # Assign default attribute to indicate points inside
         poly_attr[DEFAULT_ATTRIBUTE] = True
@@ -530,8 +503,7 @@ def interpolate_polygon_points(source, target,
 
 
 def interpolate_polygon_lines(source, target,
-                              layer_name=None,
-                              attribute_name=None):
+                              layer_name=None):
     """Interpolate from polygon vector layer to line vector data
 
     Args:
@@ -539,8 +511,6 @@ def interpolate_polygon_lines(source, target,
         * target: Vector data set (lines)
         * layer_name: Optional name of returned interpolated layer.
               If None the name of target is used for the returned layer.
-        * attribute_name: Name for new attribute.
-              If None (default) the name of source is used
 
     Returns:
         Vector data set of lines inside polygons
@@ -550,10 +520,6 @@ def interpolate_polygon_lines(source, target,
            Lines not in any polygon are ignored.
     """
 
-    # Remove?
-    if attribute_name is None:
-        attribute_name = source.get_name()
-
     # Extract line features
     lines = target.get_geometry()
     line_attributes = target.get_data()
